@@ -1,46 +1,10 @@
 /* -*- Mode: Java; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* vim:set ts=2 sw=2 sts=2 et: */
-/* ***** BEGIN LICENSE BLOCK *****
- * Version: MPL 1.1/GPL 2.0/LGPL 2.1
- *
- * The contents of this file are subject to the Mozilla Public License Version
- * 1.1 (the "License"); you may not use this file except in compliance with
- * the License. You may obtain a copy of the License at
- * http://www.mozilla.org/MPL/
- *
- * Software distributed under the License is distributed on an "AS IS" basis,
- * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
- * for the specific language governing rights and limitations under the
- * License.
- *
- * The Original Code is Jetpack.
- *
- * The Initial Developer of the Original Code is
- * the Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2010
- * the Initial Developer. All Rights Reserved.
- *
- * Contributor(s):
- *   Dietrich Ayala <dietrich@mozilla.com> (Original Author)
- *   Drew Willcoxon <adw@mozilla.com>
- *   Irakli Gozalishvili <gozala@mozilla.com>
- *   Alexandre Poirot <apoirot@mozilla.com>
- *
- * Alternatively, the contents of this file may be used under the terms of
- * either the GNU General Public License Version 2 or later (the "GPL"), or
- * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
- * in which case the provisions of the GPL or the LGPL are applicable instead
- * of those above. If you wish to allow use of your version of this file only
- * under the terms of either the GPL or the LGPL, and not to allow others to
- * use your version of this file under the terms of the MPL, indicate your
- * decision by deleting the provisions above and replace them with the notice
- * and other provisions required by the GPL or the LGPL. If you do not delete
- * the provisions above, a recipient may use your version of this file under
- * the terms of any one of the MPL, the GPL or the LGPL.
- *
- * ***** END LICENSE BLOCK ***** */
+/* This Source Code Form is subject to the terms of the Mozilla Public
+ * License, v. 2.0. If a copy of the MPL was not distributed with this
+ * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
-const {Cc, Ci} = require("chrome");
+"use strict";
 
 // Widget content types
 const CONTENT_TYPE_URI    = 1;
@@ -62,7 +26,7 @@ const EVENTS = {
   "mouseout": "mouseout",
 };
 
-if (!require("xul-app").is("Firefox")) {
+if (!require("api-utils/xul-app").is("Firefox")) {
   throw new Error([
     "The widget module currently supports only Firefox.  In the future ",
     "it will support other applications. Please see ",
@@ -70,16 +34,17 @@ if (!require("xul-app").is("Firefox")) {
   ].join(""));
 }
 
-const { validateOptions } = require("api-utils");
-const panels = require("panel");
-const { EventEmitter, EventEmitterTrait } = require("events");
-const { Trait } = require("traits");
-const LightTrait = require('light-traits').Trait;
-const { Loader, Symbiont } = require("content");
-const timer = require("timer");
-const { Cortex } = require('cortex');
-const windowsAPI = require("windows");
-const unload = require("unload");
+const { validateOptions } = require("api-utils/api-utils");
+const panels = require("./panel");
+const { EventEmitter, EventEmitterTrait } = require("api-utils/events");
+const { Trait } = require("api-utils/traits");
+const LightTrait = require('api-utils/light-traits').Trait;
+const { Loader, Symbiont } = require("api-utils/content");
+const { Cortex } = require('api-utils/cortex');
+const windowsAPI = require("./windows");
+const { setTimeout } = require("api-utils/timer");
+const unload = require("api-utils/unload");
+const { uuid } = require("api-utils/uuid");
 
 // Data types definition
 const valid = {
@@ -108,6 +73,14 @@ const valid = {
     },
     defaultValue: 16
   },
+  allow: {
+    is: ["null", "undefined", "object"],
+    map: function (v) {
+      if (!v) v = { script: true };
+      return v;
+    },
+    get defaultValue() ({ script: true })
+  },
 };
 
 // Widgets attributes definition
@@ -117,12 +90,13 @@ let widgetAttributes = {
   tooltip: valid.string,
   width: valid.width,
   content: valid.string,
-  panel: valid.panel
+  panel: valid.panel,
+  allow: valid.allow
 };
 
 // Import data definitions from loader, but don't compose with it as Model
 // functions allow us to recreate easily all Loader code.
-let loaderAttributes = require("content/loader").validationAttributes;
+let loaderAttributes = require("api-utils/content/loader").validationAttributes;
 for (let i in loaderAttributes)
   widgetAttributes[i] = loaderAttributes[i];
 
@@ -513,7 +487,7 @@ let browserManager = {
   // that calling this method can cause onTrack to be called immediately if
   // there are open windows.
   init: function () {
-    let windowTracker = new (require("window-utils").WindowTracker)(this);
+    let windowTracker = new (require("api-utils/window-utils").WindowTracker)(this);
     unload.ensure(windowTracker);
   },
 
@@ -666,11 +640,16 @@ BrowserWindow.prototype = {
     
     // Finally insert our widget in the right toolbar and in the right position
     container.insertItem(id, nextNode, null, false);
-    
-    // Update DOM in order to save position if we remove/readd the widget
-    container.setAttribute("currentset", container.currentSet);
-    // Save DOM attribute in order to save position on new window opened
-    this.window.document.persist(container.id, "currentset");
+
+    // Update DOM in order to save position: which toolbar, and which position 
+    // in this toolbar. But only do this the first time we add it to the toolbar
+    // Otherwise, this code will collide with other instance of Widget module
+    // during Firefox startup. See bug 685929.
+    if (ids.indexOf(id) == -1) {
+      container.setAttribute("currentset", container.currentSet);
+      // Save DOM attribute in order to save position on new window opened
+      this.window.document.persist(container.id, "currentset");
+    }
   }
 }
 
@@ -719,7 +698,7 @@ WidgetChrome.prototype.update = function WC_update(updatedItem, property, value)
 WidgetChrome.prototype._createNode = function WC__createNode() {
   // XUL element container for widget
   let node = this._doc.createElement("toolbaritem");
-  let guid = require("xpcom").makeUuid().toString();
+  let guid = String(uuid());
   
   // Temporary work around require("self") failing on unit-test execution ...
   let jetpackID = "testID";
@@ -733,6 +712,8 @@ WidgetChrome.prototype._createNode = function WC__createNode() {
   node.setAttribute("label", this._widget.label);
   node.setAttribute("tooltiptext", this._widget.tooltip);
   node.setAttribute("align", "center");
+  // Bug 626326: Prevent customize toolbar context menu to appear
+  node.setAttribute("context", "");
 
   // TODO move into a stylesheet, configurable by consumers.
   // Either widget.style, exposing the style object, or a URL
@@ -787,14 +768,14 @@ WidgetChrome.prototype.setContent = function WC_setContent() {
 
   switch (type) {
     case CONTENT_TYPE_HTML:
-      contentURL = "data:text/html," + encodeURIComponent(this._widget.content);
+      contentURL = "data:text/html;charset=utf-8," + encodeURIComponent(this._widget.content);
       break;
     case CONTENT_TYPE_URI:
       contentURL = this._widget.contentURL;
       break;
     case CONTENT_TYPE_IMAGE:
       let imageURL = this._widget.contentURL;
-      contentURL = "data:text/html,<html><body><img src='" +
+      contentURL = "data:text/html;charset=utf-8,<html><body><img src='" +
                    encodeURI(imageURL) + "'></body></html>";
       break;
     default:
@@ -821,9 +802,10 @@ WidgetChrome.prototype.setContent = function WC_setContent() {
     contentScriptFile: this._widget.contentScriptFile,
     contentScript: this._widget.contentScript,
     contentScriptWhen: this._widget.contentScriptWhen,
+    contentScriptOptions: this._widget.contentScriptOptions,
     allow: this._widget.allow,
     onMessage: function(message) {
-      timer.setTimeout(function() {
+      setTimeout(function() {
         self._widget._onEvent("message", message);
       }, 0);
     }
@@ -853,14 +835,14 @@ WidgetChrome.prototype.addEventHandlers = function WC_addEventHandlers() {
       return;
 
     // Proxy event to the widget
-    timer.setTimeout(function() {
+    setTimeout(function() {
       self._widget._onEvent(EVENTS[e.type], null, self.node);
     }, 0);
   };
 
   this.eventListeners = {};
   let iframe = this.node.firstElementChild;
-  for (let [type, method] in Iterator(EVENTS)) {
+  for (let type in EVENTS) {
     iframe.addEventListener(type, listener, true, true);
 
     // Store listeners for later removal
@@ -869,8 +851,8 @@ WidgetChrome.prototype.addEventHandlers = function WC_addEventHandlers() {
   
   // On document load, make modifications required for nice default
   // presentation.
-  let self = this;
   function loadListener(e) {
+    let containerStyle = self.window.getComputedStyle(self.node.parentNode);
     // Ignore event firings that target the iframe
     if (e.target == iframe)
       return;
@@ -890,6 +872,12 @@ WidgetChrome.prototype.addEventHandlers = function WC_addEventHandlers() {
       doc.body.firstElementChild.style.height = "16px";
     }
 
+    // Extend the add-on bar's default text styles to the widget.
+    doc.body.style.color = containerStyle.color;
+    doc.body.style.fontFamily = containerStyle.fontFamily;
+    doc.body.style.fontSize = containerStyle.fontSize;
+    doc.body.style.fontWeight = containerStyle.fontWeight;
+    doc.body.style.textShadow = containerStyle.textShadow;
     // Allow all content to fill the box by default.
     doc.body.style.margin = "0";
   }
@@ -918,8 +906,10 @@ WidgetChrome.prototype.addEventHandlers = function WC_addEventHandlers() {
 // Remove and unregister the widget from everything
 WidgetChrome.prototype.destroy = function WC_destroy(removedItems) {
   // remove event listeners
-  for (let [type, listener] in Iterator(this.eventListeners))
+  for (let type in this.eventListeners) {
+    let listener = this.eventListeners[type];
     this.node.firstElementChild.removeEventListener(type, listener, true);
+  }
   // remove dom node
   this.node.parentNode.removeChild(this.node);
   // cleanup symbiont
